/*
 *  spherelight.sl - light shader simulating a spherical light using a point light
 *
 * PARAMETERS
 *   intensity - scalar light intensity
 *   lightcolor - light color
 *   lightcolor_spd - light spectrum from file
 *
 *   lightradius - radius of spherical light
 *
 *   falloff - controls intensity falloff with distance
 *      0 = no falloff with distance
 *      1 = linear falloff with distance
 *      2 = square-law falloff with distance (physically correct)
 *
 *   shadowname - set to "shadow" for ray-traced shadows
 *   shadowbias - offset for shadow ray to prevent incorrect self-shadowing
 *   shadowcolor - the shadow doesn't have to be black
 *
 *   shadowgroups - groups to test with shadow rays
 *   shadowattribute - name of user attribute to query for
 *     per-object shadow groups override
 *
 *   __shadow - stores the result of the shadow query (if any)
 *   __unshadowed_Cl - stores emitted light w/o shadows
 *   __nondiffuse - set to 1 for a highlights-only light
 *   __nonspecular - set to 1 to prevent highlights from this light
 *   __foglight - set to 0 to exclude from volumes
 *
 * HISTORY
 *   03-23-2010:  created from pointlight.sl by Scott Iverson
 *
 *  (c) 2010-2012 SiTex Graphics, Inc.
 */


 #define nsamples 8

 float spherefactor(float maxangle; float radius; float Llen) {

   if ((radius<=0)||(maxangle<=0)) return 1;
   float sum = 0;

   float i;
   float lastangle = 0;
   float angle = 0;
   for (i=0; i<nsamples; i+=1) {
      angle += maxangle/nsamples;
      float a = (angle+lastangle)*0.5;
      float cosa=cos(a);
      float sina=sin(a);
      vector l=vector(-cosa,sina,0);
      vector v=normalize(vector(Llen-radius*cosa,radius*sina,0));
      float dotp = abs(l.v);
      sum += (angle*angle-lastangle*lastangle)*dotp;
      lastangle=angle;
   }
   return sum/4;
 }


light spherelight(
   float intensity = 1;
   color lightcolor = 1;
   string lightcolor_spd = "";
   float lightradius = 1;

   point from = point "shader" (0, 0, 0);

   float falloff = 2;  /* exponent for intensity falloff with distance */

   string shadowname = "";  /* shadow map name or "raytrace" for traced shadows */
   float shadowbias = 0.01;
   float shadowblur = 0.01;
   float shadowsamples = 4; /* samples or rays */

   color shadowcolor = color(0,0,0);
   string shadowgroups = "";
   string shadowattribute = "user:shadowgroups";

   string __category = "";

   output varying color __shadow = color(0,0,0);
   output varying color __unshadowed_Cl = color(0,0,0);
   output float __nondiffuse = 0;  /* set to 1 to exclude from diffuse light */
   output float __nonspecular = 0; /* set to 1 to exclude from highlights */
   output float __foglight = 1;    /* set to 0 to exclude from volumes */
   output float __channel = -1;    /* select per-light output channel */
)
{
   illuminate (from) {

      float blur = shadowblur;
      float shadowangle = 0;
      float LdotL = L.L;
      float lenL = max(sqrt(LdotL),lightradius*1.01);
      if ((lightradius>0)&&(shadowname=="raytrace")) {
         shadowangle = abs(atan(lightradius,sqrt(lenL*lenL-lightradius*lightradius ) ));
         blur = max(shadowblur,shadowangle);
         //if (shadowname!="raytrace") blur=blur/PI;
      }

      Cl = intensity * spherefactor(PI/2-shadowangle,lightradius,lenL) / pow(max(lenL,lightradius),falloff);

      color inshadow=0;
      if (shadowname!="") {
         attribute(shadowattribute, shadowgroups);
         if (shadowgroups=="") {
            inshadow = shadow(shadowname, Ps, "samples", shadowsamples,
                           "blur", blur, "bias", shadowbias, "weight", Cl);
         } else {
            if (shadowgroups=="null") {
              inshadow = 0;
            } else {
              inshadow = shadow(shadowname, Ps, "samples", shadowsamples,
                           "blur", blur, "bias", shadowbias, "weight", Cl,
                           "subset", shadowgroups);
            }
         }
      }
      __shadow=inshadow;

      __unshadowed_Cl = Cl * lightcolor;
      Cl *= mix(lightcolor, shadowcolor, inshadow);
   }
}
